#/bin/sh

##########################################################################
# Shell script to set permissions on the IMDC install folder and files.
# This function is meant to be called as follows:
#
# imsetperms DEBIAN		<PREFIX>	Debian 3.0, Debian 3.1, Ubuntu
# imsetperms REDHAT		<PREFIX>	All Redhat flavors, SuSE, Mandrake
# imsetperms FREEBSD	<PREFIX>	FreeBSD 4 and 5
# imsetperms SOLARIS	<PREFIX>	Solaris SPARC and Intel
# imsetperms MACOSX		<PREFIX>	Mac OS X
#
# Where <PREFIX> is the install location (i.e. /usr/local/imdc)
#
# The script will set permissions on all folders to 0555.  Most files will
# be set to 0440, except executable files, which are set to 0550.  The
# 'log' and 'config' folders and contents are set to 0775.
#
# On Solaris, the user is called 'intermap' to stay within 8 characters.
##########################################################################

imsetperms()
{
	# Get the InterMapper user/group name

	imuname="intermapper"
	if [ "$1" = "SOLARIS" ]; then
		imuname="intermap"
		imugroup="intermap"
	fi

	# Get the Super-user user/group name

	sugroup="wheel"
	if [ "$1" = "DEBIAN" ]; then
		sugroup="0"
	elif [ "$1" = "SOLARIS" ]; then
		sugroup="0"
	fi

	# Check to make sure the prefix folder exists

	prefix=$2
	if [ -f "$prefix" ]; then
		echo "You must pass a second argument that is a directory."
		echo "imsetperms <OS> <PREFIX>"
		return
	fi

	# First, set ownership on all files to root:wheel

	chown -R root:$sugroup "$prefix"

	# As of 5.3a3 2010-03-17, the config & log folders are no longer included in the
	# actual install on any platforms.  Instead, we'll creat them here, prior to setting
	# their permissions.  This addresses the problem where some package managers try to
	# un-install them without respecting the fact that they may be symlinks; see #81076.

	if [ ! -d "$prefix/config" ]; then
		mkdir -p "$prefix/config"
	fi

	if [ ! -d "$prefix/log" ]; then
		mkdir -p "$prefix/log"
	fi

	# Now, set ownership on mutable directories to intermapper:intermapper

	chown -R $imuname:$imuname "$prefix/config"
	chown -R $imuname:$imuname "$prefix/log"

	# Next, set permissions on files/folders recursively.
	# We need to pipe this to 'read' to deal with dirs that have spaces in
	# their names.  That prevents them from being handled by a normal FOR.

	# Start by changing all files to 0444 and all folders to 0555

	find "$prefix" | while read fname
	do
		if [ -d "$fname" ]; then
			chmod 0555 "$fname"
		else
			chmod 0444 "$fname"
		fi
	done

	# Give individual executable files ug execute permissions

	chmod 0555 "$prefix"/core/postgresql/bin/*
	chmod 0555 "$prefix"/core/python/bin/*
	chmod 0555 "$prefix"/sbin/*

	# Now, change mutable files to 0660 and mutable folders to 0770

	find "$prefix/config" "$prefix/log" | while read fname
	do
		if [ -d "$fname" ]; then
			chmod 0770 "$fname"
		else
			chmod 0660 "$fname"
		fi
	done

	# Change existing internal DB folder back to 0700 so PGSQL doesn't throw a tantrum

	if [ -d "$prefix/config/database" ]; then
		chmod 0700 "$prefix/config/database"
	fi

	# Set security contexts on shared libraries for systems using SELinux.  We first want
	# to check whether SELinux is enabled, because otherwise, even if chcon exists, it
	# will sometimes segfault when run.  On RHEL4 (and earlier, probably), chcon doesn't
	# accept the textrel_shlib_t context, so we'll just use shlib_t with different syntax.

	# Fedora 8 needs "textrel_shlib_t".   --spr 2008/03/17

	if [ -x "/usr/sbin/getenforce" ]; then
		if [ `/usr/sbin/getenforce` = "Enabled" -o `/usr/sbin/getenforce` = "Enforcing" ]; then
			if [ -x "/usr/bin/chcon" ]; then
				if /usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so >& /dev/null; then
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
					/usr/bin/chcon system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					if [ -x /usr/sbin/semanage ] ; then
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
						/usr/sbin/semanage fcontext -a -t system_u:object_r:textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					fi
				elif /usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so >& /dev/null; then
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
					/usr/bin/chcon -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					if [ -x /usr/sbin/semanage ] ; then
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
						/usr/sbin/semanage fcontext -a -t textrel_shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					fi
				else
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
					/usr/bin/chcon -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					if [ -x /usr/sbin/semanage ] ; then
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/psycopg2/_psycopg.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/crypto.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/rand.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/site-packages/OpenSSL/SSL.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_hashlib.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/_ssl.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/zlib.so
						/usr/sbin/semanage fcontext -a -t shlib_t "$prefix"/core/python/lib/python2.7/lib-dynload/binascii.so
					fi
				fi
			fi
		fi
	fi

}

imsetperms "$1" "$2"
